@@ -34,6 +34,7 @@
 #include "qgsgeometry.h"
 #include "qgstemporalrangeobject.h"
 #include "qgsmapclippingregion.h"
+#include "qgsvectorsimplifymethod.h"
 
 class QPainter;
 
@@ -204,6 +205,24 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
      */
     void setOutputDpi( double dpi );
 
+    /**
+     * Returns the target DPI (dots per inch) to be taken into consideration when rendering.
+     *
+     * The default value is -1, which states no DPI target is provided.
+     *
+     * \see setDpiTarget()
+     * \since QGIS 3.20
+     */
+    double dpiTarget() const;
+
+    /**
+     * Sets the target \a dpi (dots per inch) to be taken into consideration when rendering.
+     *
+     * \see dpiTarget()
+     * \since QGIS 3.20
+     */
+    void setDpiTarget( double dpi );
+
     /**
      * Set the magnification factor.
      * \param factor the factor of magnification
@@ -225,20 +244,26 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
      *
      * The layers are stored in the reverse order of how they are rendered (layer with index 0 will be on top).
      *
+     * Since QGIS 3.24, if the \a expandGroupLayers option is TRUE then group layers will be converted to
+     * all their child layers.
+     *
      * \see layers()
      * \see setLayers()
      */
-    QStringList layerIds() const;
+    QStringList layerIds( bool expandGroupLayers = false ) const;
 
     /**
      * Returns the list of layers which will be rendered in the map.
      *
      * The layers are stored in the reverse order of how they are rendered (layer with index 0 will be on top)
      *
+     * Since QGIS 3.24, if the \a expandGroupLayers option is TRUE then group layers will be converted to
+     * all their child layers.
+     *
      * \see setLayers()
      * \see layerIds()
      */
-    QList<QgsMapLayer *> layers() const;
+    QList<QgsMapLayer *> layers( bool expandGroupLayers = false ) const;
 
     /**
      * Sets the list of \a layers to render in the map.
@@ -376,35 +401,14 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
      */
     QColor selectionColor() const { return mSelectionColor; }
 
-    //! Enumeration of flags that adjust the way the map is rendered
-    enum Flag
-    {
-      Antialiasing             = 0x01,  //!< Enable anti-aliasing for map rendering
-      DrawEditingInfo          = 0x02,  //!< Enable drawing of vertex markers for layers in editing mode
-      ForceVectorOutput        = 0x04,  //!< Vector graphics should not be cached and drawn as raster images
-      UseAdvancedEffects       = 0x08,  //!< Enable layer opacity and blending effects
-      DrawLabeling             = 0x10,  //!< Enable drawing of labels on top of the map
-      UseRenderingOptimization = 0x20,  //!< Enable vector simplification and other rendering optimizations
-      DrawSelection            = 0x40,  //!< Whether vector selections should be shown in the rendered map
-      DrawSymbolBounds         = 0x80,  //!< Draw bounds of symbols (for debugging/testing)
-      RenderMapTile            = 0x100, //!< Draw map such that there are no problems between adjacent tiles
-      RenderPartialOutput      = 0x200, //!< Whether to make extra effort to update map image with partially rendered layers (better for interactive map canvas). Added in QGIS 3.0
-      RenderPreviewJob         = 0x400, //!< Render is a 'canvas preview' render, and shortcuts should be taken to ensure fast rendering
-      RenderBlocking           = 0x800, //!< Render and load remote sources in the same thread to ensure rendering remote sources (svg and images). WARNING: this flag must NEVER be used from GUI based applications (like the main QGIS application) or crashes will result. Only for use in external scripts or QGIS server.
-      LosslessImageRendering   = 0x1000, //!< Render images losslessly whenever possible, instead of the default lossy jpeg rendering used for some destination devices (e.g. PDF). This flag only works with builds based on Qt 5.13 or later.
-      Render3DMap              = 0x2000, //!< Render is for a 3D map
-      // TODO: ignore scale-based visibility (overview)
-    };
-    Q_DECLARE_FLAGS( Flags, Flag )
-
     //! Sets combination of flags that will be used for rendering
-    void setFlags( QgsMapSettings::Flags flags );
+    void setFlags( Qgis::MapSettingsFlags flags );
     //! Enable or disable a particular flag (other flags are not affected)
-    void setFlag( Flag flag, bool on = true );
+    void setFlag( Qgis::MapSettingsFlag flag, bool on = true );
     //! Returns combination of flags used for rendering
-    Flags flags() const;
+    Qgis::MapSettingsFlags flags() const;
     //! Check whether a particular flag is enabled
-    bool testFlag( Flag flag ) const;
+    bool testFlag( Qgis::MapSettingsFlag flag ) const;
 
     /**
      * Returns the text render format, which dictates how text is rendered (e.g. as paths or real text objects).
@@ -412,7 +416,7 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
      * \see setTextRenderFormat()
      * \since QGIS 3.4.3
      */
-    QgsRenderContext::TextRenderFormat textRenderFormat() const
+    Qgis::TextRenderFormat textRenderFormat() const
     {
       return mTextRenderFormat;
     }
@@ -426,7 +430,7 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
      * \see textRenderFormat()
      * \since QGIS 3.4.3
      */
-    void setTextRenderFormat( QgsRenderContext::TextRenderFormat format )
+    void setTextRenderFormat( Qgis::TextRenderFormat format )
     {
       mTextRenderFormat = format;
       // ensure labeling engine setting is also kept in sync, just in case anyone accesses QgsMapSettings::labelingEngineSettings().defaultTextRenderFormat()
@@ -441,7 +445,7 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
 
     //! Check whether the map settings are valid and can be used for rendering
     bool hasValidSettings() const;
-    //! Returns the actual extent derived from requested extent that takes takes output image size into account
+    //! Returns the actual extent derived from requested extent that takes output image size into account
     QgsRectangle visibleExtent() const;
 
     /**
@@ -449,6 +453,14 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
      * \since QGIS 2.8
      */
     QPolygonF visiblePolygon() const;
+
+    /**
+     * Returns the visible area as a polygon (may be rotated) with extent buffer included
+     * \see extentBuffer()
+     * \since QGIS 3.22
+     */
+    QPolygonF visiblePolygonWithBuffer() const;
+
     //! Returns the distance in geographical coordinates that equals to one pixel in the map
     double mapUnitsPerPixel() const;
 
@@ -582,6 +594,28 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
      */
     QgsCoordinateTransform layerTransform( const QgsMapLayer *layer ) const;
 
+    /**
+     * \brief Compute the extent such that its \a center is at the specified
+     * position (mapped to the destinatonCrs) and the zoom factor corresponds
+     * to the specified \a scale
+     * \param center the center, in map coordinates
+     * \param scale the desired zoom factor (the x part of 1:x)
+     * \returns an extent which can be passed to QgsMapCanvas::setExtent
+     * \see computeScaleForExtent()
+     * \since QGIS 3.22
+     */
+    QgsRectangle computeExtentForScale( const QgsPointXY &center, double scale ) const;
+
+    /**
+     * \brief Compute the scale that corresponds to the specified \a extent
+     * \param extent the extent, as passed to \see QgsMapCanvas::setExtent
+     * \returns the scale denominator
+     * \see computeExtentForScale()
+     * \note This function does not consider any map rotation
+     * \since QGIS 3.22
+     */
+    double computeScaleForExtent( const QgsRectangle &extent ) const;
+
     //! returns current extent of layer set
     QgsRectangle fullExtent() const;
 
@@ -771,9 +805,69 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
      */
     void setZRange( const QgsDoubleRange &range );
 
+    /**
+     * Returns the rendering usage
+     *
+     * \see setRendererUsage()
+     * \since QGIS 3.24
+     */
+    Qgis::RendererUsage rendererUsage() const;
+
+    /**
+     * Sets the rendering usage
+     *
+     * \note This usage not alter how the map gets rendered but the intention is that data provider
+     * knows the context of rendering and may report that to the backend.
+     *
+     * \see rendererUsage()
+     * \since QGIS 3.24
+     */
+    void setRendererUsage( Qgis::RendererUsage rendererUsage );
+
+    /**
+     * Returns the frame rate of the map (in frames per second), for maps which are part of an animation.
+     *
+     * Returns -1 if the map is not associated with an animation.
+     *
+     * \see setFrameRate()
+     * \since QGIS 3.26
+     */
+    double frameRate() const;
+
+    /**
+     * Sets the frame \a rate of the map (in frames per second), for maps which are part of an animation.
+     *
+     * Defaults to -1 if the map is not associated with an animation.
+     *
+     * \see frameRate()
+     * \since QGIS 3.26
+     */
+    void setFrameRate( double rate );
+
+    /**
+     * Returns the current frame number of the map, for maps which are part of an animation.
+     *
+     * Returns -1 if the map is not associated with an animation.
+     *
+     * \see setCurrentFrame()
+     * \since QGIS 3.26
+     */
+    long long currentFrame() const;
+
+    /**
+     * Sets the current \a frame of the map, for maps which are part of an animation.
+     *
+     * Defaults to -1 if the map is not associated with an animation.
+     *
+     * \see currentFrame()
+     * \since QGIS 3.26
+     */
+    void setCurrentFrame( long long frame );
+
   protected:
 
-    double mDpi;
+    double mDpi = 96.0;
+    double mDpiTarget = -1;
 
     QSize mSize;
     float mDevicePixelRatio = 1.0;
@@ -798,7 +892,7 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
     QColor mBackgroundColor;
     QColor mSelectionColor;
 
-    Flags mFlags;
+    Qgis::MapSettingsFlags mFlags;
 
     QImage::Format mImageFormat = QImage::Format_ARGB32_Premultiplied;
 
@@ -808,8 +902,10 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
     QgsLabelingEngineSettings mLabelingEngineSettings;
 
     // derived properties
-    bool mValid = false; //!< Whether the actual settings are valid (set in updateDerived())
-    QgsRectangle mVisibleExtent; //!< Extent with some additional white space that matches the output aspect ratio
+    //! Whether the actual settings are valid (set in updateDerived())
+    bool mValid = false;
+    //! Extent with some additional white space that matches the output aspect ratio
+    QgsRectangle mVisibleExtent;
     double mMapUnitsPerPixel = 1;
     double mScale = 1;
 
@@ -821,12 +917,17 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
 
     QgsPathResolver mPathResolver;
 
-    QgsRenderContext::TextRenderFormat mTextRenderFormat = QgsRenderContext::TextFormatAlwaysOutlines;
+    Qgis::TextRenderFormat mTextRenderFormat = Qgis::TextRenderFormat::AlwaysOutlines;
 
     QgsGeometry mLabelBoundaryGeometry;
 
     QgsVectorSimplifyMethod mSimplifyMethod;
 
+    Qgis::RendererUsage mRendererUsage = Qgis::RendererUsage::Unknown;
+
+    double mFrameRate = -1;
+    long long mCurrentFrame = -1;
+
 #ifdef QGISDEBUG
     bool mHasTransformContext = false;
 #endif
@@ -843,7 +944,4 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
 
 };
 
-Q_DECLARE_OPERATORS_FOR_FLAGS( QgsMapSettings::Flags )
-
-
 #endif // QGSMAPSETTINGS_H
