@@ -23,12 +23,12 @@
 #include "qgis_sip.h"
 #include "qgsfields.h"
 #include "qgsfeedback.h"
-#include "qgssymbol.h"
 #include "qgstaskmanager.h"
 #include "qgsogrutils.h"
 #include "qgsrenderer.h"
 #include "qgsgeometryengine.h"
 #include "qgsfeaturesink.h"
+#include "qgsrendercontext.h"
 #include <ogr_api.h>
 
 class QgsSymbolLayer;
@@ -37,10 +37,12 @@ class QgsFeatureIterator;
 
 /**
  * \ingroup core
-  * A convenience class for writing vector files to disk.
- There are two possibilities how to use this class:
- 1. static call to QgsVectorFileWriter::writeAsVectorFormat(...) which saves the whole vector layer
- 2. create an instance of the class and issue calls to addFeature(...)
+ * \brief A convenience class for writing vector layers to disk based formats (e.g. Shapefiles, GeoPackage).
+ *
+ * There are two possibilities how to use this class:
+ *
+ * 1. A static call to QgsVectorFileWriter::writeAsVectorFormat(...) which saves the whole vector layer.
+ * 2. Create an instance of the class and issue calls to addFeature(...).
  */
 class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 {
@@ -175,6 +177,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
       ErrProjection,
       ErrFeatureWriteFailed,
       ErrInvalidLayer,
+      ErrSavingMetadata, //!< Metadata saving failed
       Canceled, //!< Writing was interrupted by manual cancellation
     };
 
@@ -185,6 +188,16 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
       SymbolLayerSymbology //Exports one feature per symbol layer (considering symbol levels)
     };
 
+    /**
+     * Source for exported field names.
+     *
+     * \since QGIS 3.18
+     */
+    enum FieldNameSource
+    {
+      Original = 0, //!< Use original field names
+      PreferAlias, //!< Use the field alias as the exported field name, wherever one is set. Otherwise use the original field names.
+    };
 
     /**
      * Options for sorting and filtering vector formats.
@@ -199,7 +212,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
     /**
      * \ingroup core
-     * Interface to convert raw field values to their user-friendly value.
+     * \brief Interface to convert raw field values to their user-friendly value.
      * \since QGIS 2.16
      */
     class CORE_EXPORT FieldValueConverter
@@ -439,7 +452,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
     /**
      * \ingroup core
-     * Options to pass to writeAsVectorFormat()
+     * \brief Options to pass to writeAsVectorFormat()
      * \since QGIS 3.0
      */
     class CORE_EXPORT SaveVectorOptions
@@ -483,6 +496,9 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
         //! Attributes to export (empty means all unless skipAttributeCreation is set)
         QgsAttributeList attributes;
 
+        //! Attributes export names
+        QStringList attributesExportNames;
+
         //! Symbology to export
         QgsVectorFileWriter::SymbologyExport symbologyExport = NoSymbology;
 
@@ -514,6 +530,29 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
         //! Optional feedback object allowing cancellation of layer save
         QgsFeedback *feedback = nullptr;
+
+        /**
+         * Source for exported field names.
+         *
+         * \since QGIS 3.18
+         */
+        FieldNameSource fieldNameSource = Original;
+
+        /**
+         * Set to TRUE to save layer metadata for the exported vector file.
+         *
+         * \see layerMetadata
+         * \since QGIS 3.20
+         */
+        bool saveMetadata = false;
+
+        /**
+         * Layer metadata to save for the exported vector file. This will only be used if saveMetadata is TRUE.
+         *
+         * \see saveMetadata
+         * \since QGIS 3.20
+         */
+        QgsLayerMetadata layerMetadata;
     };
 
 #ifndef SIP_RUN
@@ -570,7 +609,8 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
                                            QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags()
 #ifndef SIP_RUN
                                                , QString *newLayer = nullptr,
-                                           QgsCoordinateTransformContext transformContext = QgsCoordinateTransformContext()
+                                           const QgsCoordinateTransformContext &transformContext = QgsCoordinateTransformContext(),
+                                           FieldNameSource fieldNameSource = Original
 #endif
                                          ) SIP_DEPRECATED;
 
@@ -592,6 +632,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
      * \param newLayer potentially modified layer name (output parameter) (added in QGIS 3.4)
      * \param transformContext transform context, needed if the output file srs is forced to specific crs (added in QGIS 3.10.3)
      * \param sinkFlags feature sink flags (added in QGIS 3.10.3)
+     * \param fieldNameSource source for field names (since QGIS 3.18)
      * \note not available in Python bindings
      * \deprecated Use create() instead.
      */
@@ -609,8 +650,9 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
                                            const QString &layerName,
                                            QgsVectorFileWriter::ActionOnExistingFile action,
                                            QString *newLayer = nullptr,
-                                           QgsCoordinateTransformContext transformContext = QgsCoordinateTransformContext(),
-                                           QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags()
+                                           const QgsCoordinateTransformContext &transformContext = QgsCoordinateTransformContext(),
+                                           QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags(),
+                                           FieldNameSource fieldNameSource = Original
                                          ) SIP_SKIP;
 
     //! QgsVectorFileWriter cannot be copied.
@@ -651,15 +693,35 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
      * \param newLayer potentially modified layer name (output parameter)
      * \param errorMessage will be set to the error message text, if an error occurs while writing the layer
      * \returns Error message code, or QgsVectorFileWriter.NoError if the write operation was successful
-     * \since QGIS 3.10.3
+     * \deprecated since QGIS 3.20, use writeAsVectorFormatV3 instead
      */
-    static QgsVectorFileWriter::WriterError writeAsVectorFormatV2( QgsVectorLayer *layer,
+    Q_DECL_DEPRECATED static QgsVectorFileWriter::WriterError writeAsVectorFormatV2( QgsVectorLayer *layer,
         const QString &fileName,
         const QgsCoordinateTransformContext &transformContext,
         const QgsVectorFileWriter::SaveVectorOptions &options,
         QString *newFilename = nullptr,
         QString *newLayer = nullptr,
-        QString *errorMessage SIP_OUT = nullptr );
+        QString *errorMessage SIP_OUT = nullptr ) SIP_DEPRECATED;
+
+    /**
+     * Writes a layer out to a vector file.
+     * \param layer source layer to write
+     * \param fileName file name to write to
+     * \param transformContext coordinate transform context
+     * \param options save options
+     * \param newFilename potentially modified file name (output parameter)
+     * \param newLayer potentially modified layer name (output parameter)
+     * \param errorMessage will be set to the error message text, if an error occurs while writing the layer
+     * \returns Error message code, or QgsVectorFileWriter.NoError if the write operation was successful
+     * \since QGIS 3.20
+     */
+    static QgsVectorFileWriter::WriterError writeAsVectorFormatV3( QgsVectorLayer *layer,
+        const QString &fileName,
+        const QgsCoordinateTransformContext &transformContext,
+        const QgsVectorFileWriter::SaveVectorOptions &options,
+        QString *errorMessage SIP_OUT = nullptr,
+        QString *newFilename SIP_OUT = nullptr,
+        QString *newLayer SIP_OUT = nullptr );
 
     /**
      * Details of available filters and formats.
@@ -759,10 +821,10 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     static QString convertCodecNameForEncodingOption( const QString &codecName );
 
     //! Checks whether there were any errors in constructor
-    QgsVectorFileWriter::WriterError hasError();
+    QgsVectorFileWriter::WriterError hasError() const;
 
     //! Retrieves error message
-    QString errorMessage();
+    QString errorMessage() const;
 
     bool addFeature( QgsFeature &feature, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
     bool addFeatures( QgsFeatureList &features, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
@@ -775,7 +837,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     bool addFeatureWithStyle( QgsFeature &feature, QgsFeatureRenderer *renderer, QgsUnitTypes::DistanceUnit outputUnit = QgsUnitTypes::DistanceMeters );
 
     //! \note not available in Python bindings
-    QMap<int, int> attrIdxToOgrIdx() { return mAttrIdxToOgrIdx; } SIP_SKIP
+    QMap<int, int> attrIdxToOgrIdx() const { return mAttrIdxToOgrIdx; } SIP_SKIP
 
     //! Close opened shapefile for writing
     ~QgsVectorFileWriter() override;
@@ -900,7 +962,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
       QgsWkbTypes::Type sourceWkbType = QgsWkbTypes::Unknown;
       QgsFields sourceFields;
       QString providerType;
-      long featureCount = 0;
+      long long featureCount = 0;
       QgsFeatureIds selectedFeatureIds;
       QString dataSourceUri;
       QString storageType;
@@ -967,7 +1029,8 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
                QgsVectorFileWriter::FieldValueConverter *fieldValueConverter,
                const QString &layerName,
                QgsVectorFileWriter::ActionOnExistingFile action, QString *newLayer, QgsFeatureSink::SinkFlags sinkFlags,
-               const QgsCoordinateTransformContext &transformContext );
+               const QgsCoordinateTransformContext &transformContext,
+               FieldNameSource fieldNameSource );
     void resetMap( const QgsAttributeList &attributes );
 
     std::unique_ptr< QgsFeatureRenderer > mRenderer;
@@ -977,7 +1040,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     std::unique_ptr< QgsCoordinateTransform > mCoordinateTransform;
 
     bool mUsingTransaction = false;
-    bool supportsStringList = false;
+    QSet< QVariant::Type > mSupportedListSubTypes;
 
     void createSymbolLayerTable( QgsVectorLayer *vl, const QgsCoordinateTransform &ct, OGRDataSourceH ds );
     gdal::ogr_feature_unique_ptr createFeature( const QgsFeature &feature );
