@@ -24,6 +24,7 @@
 #include "qgsgeometry.h"
 
 class QgsVectorLayer;
+class QgsVectorLayerEditBufferGroup;
 
 typedef QList<int> QgsAttributeList SIP_SKIP;
 typedef QSet<int> QgsAttributeIds SIP_SKIP;
@@ -42,7 +43,6 @@ class CORE_EXPORT QgsVectorLayerEditBuffer : public QObject
     //! Returns TRUE if the provider has been modified since the last commit
     virtual bool isModified() const;
 
-
     /**
      * Adds a feature
      * \param f feature to add
@@ -73,12 +73,12 @@ class CORE_EXPORT QgsVectorLayerEditBuffer : public QObject
     virtual bool changeAttributeValues( QgsFeatureId fid, const QgsAttributeMap &newValues, const QgsAttributeMap &oldValues );
 
     /**
-     * Add an attribute field (but does not commit it)
-        returns true if the field was added
+     * Adds an attribute field (but does not commit it)
+     * returns TRUE if the field was added
     */
     virtual bool addAttribute( const QgsField &field );
 
-    //! Delete an attribute field (but does not commit it)
+    //! Deletes an attribute field (but does not commit it)
     virtual bool deleteAttribute( int attr );
 
     /**
@@ -90,19 +90,19 @@ class CORE_EXPORT QgsVectorLayerEditBuffer : public QObject
     virtual bool renameAttribute( int attr, const QString &newName );
 
     /**
-      Attempts to commit any changes to disk.  Returns the result of the attempt.
-      If a commit fails, the in-memory changes are left alone.
-
-      This allows editing to continue if the commit failed on e.g. a
-      disallowed value in a Postgres database - the user can re-edit and try
-      again.
-
-      The commits occur in distinct stages,
-      (add attributes, add features, change attribute values, change
-      geometries, delete features, delete attributes)
-      so if a stage fails, it's difficult to roll back cleanly.
-      Therefore any error message also includes which stage failed so
-      that the user has some chance of repairing the damage cleanly.
+     * Attempts to commit any changes to disk.  Returns the result of the attempt.
+     * If a commit fails, the in-memory changes are left alone.
+     *
+     * This allows editing to continue if the commit failed on e.g. a
+     * disallowed value in a Postgres database - the user can re-edit and try
+     * again.
+     *
+     * The commits occur in distinct stages,
+     * (add attributes, add features, change attribute values, change
+     * geometries, delete features, delete attributes)
+     * so if a stage fails, it's difficult to roll back cleanly.
+     * Therefore any error message also includes which stage failed so
+     * that the user has some chance of repairing the damage cleanly.
      */
     virtual bool commitChanges( QStringList &commitErrors );
 
@@ -115,6 +115,14 @@ class CORE_EXPORT QgsVectorLayerEditBuffer : public QObject
     */
     QgsFeatureMap addedFeatures() const { return mAddedFeatures; }
 
+    /**
+     * Returns a list of the features IDs for all newly added or edited features
+     * in the buffer.
+     *
+     * \since QGIS 3.20
+     */
+    QgsFeatureIds allAddedOrEditedFeatures() const;
+
     /**
      * Returns TRUE if the specified feature ID has been added but not committed.
      * \param id feature ID
@@ -184,6 +192,25 @@ class CORE_EXPORT QgsVectorLayerEditBuffer : public QObject
      */
     bool isFeatureDeleted( QgsFeatureId id ) const { return mDeletedFeatureIds.contains( id ); }
 
+    /**
+     * Updates \a fields
+     * \note Not available in Python bindings
+     * \since QGIS 3.18
+     */
+    void updateFields( QgsFields &fields ) SIP_SKIP;
+
+    /**
+     * Returns the parent edit buffer group for this edit buffer, or nullptr if not part of a group.
+     * \since QGIS 3.26
+     */
+    QgsVectorLayerEditBufferGroup *editBufferGroup() const;
+
+    /**
+     * Set the parent edit buffer group for this edit buffer.
+     * \since QGIS 3.26
+     */
+    void setEditBufferGroup( QgsVectorLayerEditBufferGroup *editBufferGroup );
+
     //QString dumpEditBuffer();
 
   protected slots:
@@ -236,8 +263,6 @@ class CORE_EXPORT QgsVectorLayerEditBuffer : public QObject
     //! Constructor for QgsVectorLayerEditBuffer
     QgsVectorLayerEditBuffer() = default;
 
-    void updateFields( QgsFields &fields );
-
     //! Update feature with uncommitted geometry updates
     void updateFeatureGeometry( QgsFeature &f );
 
@@ -257,7 +282,6 @@ class CORE_EXPORT QgsVectorLayerEditBuffer : public QObject
 
   protected:
     QgsVectorLayer *L = nullptr;
-    friend class QgsVectorLayer;
 
     friend class QgsVectorLayerUndoCommand;
     friend class QgsVectorLayerUndoCommandAddFeature;
@@ -281,8 +305,8 @@ class CORE_EXPORT QgsVectorLayerEditBuffer : public QObject
 
     /**
      * Deleted feature IDs which are not committed.  Note a feature can be added and then deleted
-        again before the change is committed - in that case the added feature would be removed
-        from mAddedFeatures only and *not* entered here.
+     * again before the change is committed - in that case the added feature would be removed
+     * from mAddedFeatures only and *not* entered here.
      */
     QgsFeatureIds mDeletedFeatureIds;
 
@@ -304,7 +328,76 @@ class CORE_EXPORT QgsVectorLayerEditBuffer : public QObject
     //! Changed geometries which are not committed.
     QgsGeometryMap mChangedGeometries;
 
+    QgsVectorLayerEditBufferGroup *mEditBufferGroup = nullptr;
+
     friend class QgsGrassProvider; //GRASS provider totally abuses the edit buffer
+
+  private:
+
+    friend class QgsVectorLayerEditBufferGroup;
+
+    /**
+     * Check geometry of added features for compatibility with data provider
+     * \param commitErrors will be extended in case of error
+     * \returns TRUE if compatible
+     */
+    bool commitChangesCheckGeometryTypeCompatibility( QStringList &commitErrors );
+
+    /**
+     * Delete attributes
+     * \param attributesDeleted is set if one or more attributes where deleted
+     * \param commitErrors will be extended in case of error
+     * \returns TRUE on success
+     */
+    bool commitChangesDeleteAttributes( bool &attributesDeleted, QStringList &commitErrors );
+
+    /**
+     * Rename attributes
+     * \param attributesRenamed is set if one or more attributes where renamed
+     * \param commitErrors will be extended in case of error
+     * \returns TRUE on success
+     */
+    bool commitChangesRenameAttributes( bool &attributesRenamed, QStringList &commitErrors );
+
+    /**
+     * Add new attributes
+     * \param attributesAdded is set if one or more attributes where added
+     * \param commitErrors will be extended in case of error
+     * \returns TRUE on success
+     */
+    bool commitChangesAddAttributes( bool &attributesAdded, QStringList &commitErrors );
+
+    /**
+     * Check that delete/rename/add attributes operations where successful
+     * \param oldFields are the fields before delete/rename/add operations
+     * \param commitErrors will be extended in case of error
+     * \returns TRUE on success
+     */
+    bool commitChangesCheckAttributesModifications( const QgsFields oldFields, QStringList &commitErrors );
+
+    /**
+     * Change attributes
+     * \param attributesChanged is set if one or more attributes where changed
+     * \param commitErrors will be extended in case of error
+     * \returns TRUE on success
+     */
+    bool commitChangesChangeAttributes( bool &attributesChanged, QStringList &commitErrors );
+
+    /**
+     * Delete features
+     * \param featuresDeleted is set if one or more features where deleted
+     * \param commitErrors will be extended in case of error
+     * \returns TRUE on success
+     */
+    bool commitChangesDeleteFeatures( bool &featuresDeleted, QStringList &commitErrors );
+
+    /**
+     * Add new features
+     * \param featuresAdded is set if one or more features where deleted
+     * \param commitErrors will be extended in case of error
+     * \returns TRUE on success
+     */
+    bool commitChangesAddFeatures( bool &featuresAdded, QStringList &commitErrors );
 };
 
 #endif // QGSVECTORLAYEREDITBUFFER_H
